﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Higuchi.Net.Mail
{
	/// <summary>
    /// Represent context of request and response process and provide data about context.
    /// </summary>
    public class AsynchronousContext : IDisposable
    {
		private static BufferManager _BufferManager = null;
		private DateTime _StartTime = DateTime.Now;
		private ArraySegment<Byte> _Buffer;
		private List<Byte> _Data = new List<byte>();
        private Exception _Exception = null;
		private Boolean _Timeout = false;
		private Encoding _Encoding = Encoding.ASCII;
		private Boolean _IsDisposed = false;
		/// <summary>
		/// 
		/// </summary>
		public static BufferManager BufferManager
		{
			get
			{
				if (_BufferManager == null)
				{
					_BufferManager = new BufferManager(256, 8192);
				}
				return _BufferManager;
			}
			set { _BufferManager = value; }
		}
		internal DateTime StartTime
		{
			get { return _StartTime; }
			set { _StartTime = value; }
		}
		internal ArraySegment<Byte> Buffer
		{
			get { return _Buffer; }
		}
		/// <summary>
		/// 
		/// </summary>
		protected Encoding Encoding
		{
			get { return _Encoding; }
			set { _Encoding = value; }
		}
        /// <summary>
        /// 
        /// </summary>
		public List<Byte> Data
		{
			get { return _Data; }
		}
        /// <summary>
        /// 
        /// </summary>
        public Exception Exception
        {
            get { return _Exception; }
            set { _Exception = value; }
        }
        /// <summary>
        /// 
        /// </summary>
		public Boolean Timeout
		{
			get { return _Timeout; }
			set { _Timeout = value; }
		}
		internal AsynchronousContext(Encoding encoding)
		{
			_Encoding = encoding;
			_Buffer = AsynchronousContext.BufferManager.CheckOut();
		}
		/// 終了処理を実行し、システムリソースを解放します。
		/// <summary>
		/// dipose and release system resoures.
		/// 終了処理を実行し、システムリソースを解放します。
		/// </summary>
		public void Dispose()
		{
			GC.SuppressFinalize(this);
			this.Dispose(true);
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="disposing"></param>
		protected void Dispose(Boolean disposing)
		{
			if (disposing)
			{
				if (this._IsDisposed == false &&
					this._Buffer != null)
				{
					AsynchronousContext.BufferManager.CheckIn(this._Buffer);
					this._IsDisposed = true;
				}
			}
		}
		/// <summary>
		/// 
		/// </summary>
		~AsynchronousContext()
		{
			this.Dispose(false);
		}
    }
}
