﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using Higuchi.Net.Mail;

namespace Higuchi.Net.Pop3.Command
{
    /// UIDLコマンドを表現するクラスです。
    /// <summary>
    /// UIDLコマンドを表現するクラスです。
    /// </summary>
    public class Uidl : Pop3Command
    {
		private class RegexList
		{
			public static Regex CheckFormat = new Regex(@"^([0-9]+)[ |	]+([\x21-\x7e]+).*$", RegexOptions.None);
		}
        private Int64? _MailIndex = null;
		/// <summary>
		/// 
		/// </summary>
        public override String Name
        {
            get { return "Uidl"; }
        }
		/// <summary>
		/// 
		/// </summary>
        public Int64? MailIndex
        {
            get { return this._MailIndex; }
        }
		/// <summary>
		/// 
		/// </summary>
        public Uidl()
        {
        }
		/// <summary>
		/// 
		/// </summary>
		/// <param name="mailIndex"></param>
        public Uidl(Int64 mailIndex)
        {
            if (mailIndex < 1)
            { throw new ArgumentException(); }
            this._MailIndex = mailIndex;
        }
		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
        public override String GetCommandString()
        {
			if (this._MailIndex.HasValue == true)
			{
				return String.Format("{0} {1}", this.Name, this._MailIndex);
			}
			return this.Name;
        }
        /// 受信した行の文字列を解析し、メールのIndexを取得します。
        /// <summary>
        /// 受信した行の文字列を解析し、メールのIndexを取得します。
        /// </summary>
        /// <param name="line"></param>
        /// <returns></returns>
        public static String GetMessageIndex(String line)
        {
            return Regex.Replace(line.Replace(MailParser.NewLine, ""), @"^([0-9]+)[ |	]+.*$", "$1");
        }
        /// 受信した行の文字列を解析し、メールのUIDを取得します。
        /// <summary>
        /// 受信した行の文字列を解析し、メールのUIDを取得します。
        /// </summary>
        /// <param name="line"></param>
        /// <remarks>Characters in the range 0x21 to 0x7E: http://www.ietf.org/rfc/rfc1939.txt</remarks>
        /// <returns></returns>
        public static String GetUid(String line)
        {
			return Regex.Replace(line.Replace(MailParser.NewLine, ""), @"^[0-9]+[ |	]+([\x21-\x7E]+)$", "$1");
        }
		/// <summary>
		/// 
		/// </summary>
        public class Result
        {
            private Int64 _MailIndex = 0;
            private String _Uid = "";
			/// <summary>
			/// 
			/// </summary>
            public Int64 MailIndex
            {
                get { return this._MailIndex; }
            }
			/// <summary>
			/// 
			/// </summary>
            public String Uid
            {
                get { return this._Uid; }
            }
			/// <summary>
			/// 
			/// </summary>
			/// <param name="text"></param>
            public Result(String text)
            {
				if (text == null)
				{ throw new ArgumentNullException("text"); }

                String mailIndexString = Uidl.GetMessageIndex(text);

                this._MailIndex = Convert.ToInt64(mailIndexString);
                this._Uid = text.Substring(mailIndexString.Length).Trim();
            }
			/// <summary>
			/// 
			/// </summary>
			/// <param name="mailIndex"></param>
			/// <param name="uid"></param>
            public Result(Int64 mailIndex, String uid)
            {
                this._MailIndex = mailIndex;
                this._Uid = uid;
            }
            /// 受信した行の文字列が正しいフォーマットかどうかを示す値を取得します。
            /// <summary>
            /// 受信した行の文字列が正しいフォーマットかどうかを示す値を取得します。
            /// </summary>
            /// <param name="line"></param>
            /// <returns></returns>
            public static Boolean CheckFormat(String line)
            {
				return RegexList.CheckFormat.IsMatch(line.Replace(MailParser.NewLine, ""));
            }
        }
    }
}
