using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Configuration;
using System.Threading;
using Higuchi.Net;
using Higuchi.Net.Pop3;
using Higuchi.Net.Smtp;
using C = Higuchi.Net.Pop3.Command;

namespace MailSample
{
    public partial class Form1 : Form
    {
        public delegate void AddToListbox(Pop3Message inMessage);
        private Pop3Client zClient = null;
        private Int32 zTotalMailCount = 0;
        private Int32 zStartIndex = 0;
        private Int32 zDisplayMailCount = 20;
        private Pop3Message zMessage = null;
        private List<Pop3Message> zMessageList = new List<Pop3Message>();
		private Object _IsExecutingLock = new object();
		private Boolean _IsExecuting = false;
        public Form1()
        {
            InitializeComponent();
        }
        private void Form1_Load(object sender, EventArgs e)
        {
#if DEBUG
            this.DebugButton.Visible = true;
#endif

            this.MailList.DisplayMember = "Subject";
            this.MailList.ValueMember = "Index";
            this.AttachedFileList.DisplayMember = "FileName";
            this.AttachedFileList.ValueMember = "FileName";
            this.Initialize();
        }
        private void CreatePop3Client()
        {
			try
			{
				this.zClient = new Pop3Client(this.UserName.Text, this.Password.Text, this.ServerName.Text);
				this.zClient.Port = Int32.Parse(this.Port.Text);
				this.zClient.Ssl = this.SSL.Checked;
				this.zClient.ReceiveTimeout = 1000000;

				if (this.zClient.Authenticate() == false)
				{
					this.zClient.Close();
					MessageBox.Show("Log in failure.");
					return;
				}
			}
			catch
			{
				MessageBox.Show("Log in failure.");
				this.zClient = null;
			}
        }
        private void Initialize()
        {
            try
            {
                this.CreatePop3Client();
				if (this.zClient == null ||
					this.zClient.State != Pop3ConnectionState.Authenticated)
				{ return; }
                if (this.zStartIndex == 0)
                {
                    this.zTotalMailCount = (Int32)this.zClient.GetTotalMessageCount();
                    this.zStartIndex = this.zTotalMailCount;
                }
                this.ListMailIndex.Text = this.zStartIndex.ToString();
                this.DetailMailIndex.Text = this.zStartIndex.ToString();
            }
            finally
            {
				if (this.zClient != null)
				{
					this.zClient.Close();
					this.zClient = null;
				}
			}
			this.SetMailList();
		}
		private void ApplyButton_Click(object sender, EventArgs e)
        {
			if (this.zClient != null)
			{
				this.zClient.Close();
			}
			this.Initialize();
		}
		private void SetMailList()
        {
			lock (_IsExecutingLock)
			{
				if (_IsExecuting == true)
				{
					MessageBox.Show("Now receiving mail...");
					return;
				}
			}
			if (this.zClient != null &&
				this.zClient.Available == true)
			{
				this.zClient.Close();
			}

			this.MailList.Items.Clear();
			this.zMessageList.Clear();

			this.CreatePop3Client();
			if (this.zClient == null || this.zClient.State != Pop3ConnectionState.Authenticated || this.zStartIndex == 0)
			{ return; }
			this.ListMailIndex.Text = this.zStartIndex.ToString();

			lock (_IsExecutingLock)
			{
				_IsExecuting = true;
			}
			this.zClient.GetMessage(this.zStartIndex - this.MailList.Items.Count, this.GetMessageCallBack);
        }
        private void GetMessageCallBack(Pop3Message inMessage)
        {
			this.zMessageList.Add(inMessage);
			if (this.MailList.InvokeRequired == true)
            {
                AddToListbox md = this.SetToListbox;
                this.MailList.Invoke(md, inMessage);
            }
            else
            {
                this.SetToListbox(inMessage);
            }
        }
        private void SetToListbox(Pop3Message inMessage)
        {
            this.MailList.Items.Add(inMessage);
			if (this.MailList.Items.Count == this.zStartIndex || 
				this.MailList.Items.Count == this.zDisplayMailCount)
			{
				lock (_IsExecutingLock)
				{
					_IsExecuting = false;
				}
				this.zClient.Close();
				this.zClient = null;
			}
			else
			{
				this.zClient.GetMessage(this.zStartIndex - this.MailList.Items.Count, this.GetMessageCallBack);
			}
        }
        private void SetMailData(Pop3Message inMessage)
        {
            if (inMessage == null)
            { return; }

            Pop3Message pm = inMessage;
            List<Pop3Content> l = null;

            this.zMessage = pm;

            this.From.Text = pm.From;
            this.Date.Text = pm.Date.ToString("yyyy/MM/dd (ddd)  HH:mm:dd");
            this.To.Text = pm.To;
            this.Cc.Text = pm.Cc;
            this.Subject.Text = pm.Subject;
            this.BodyText.Text = pm.BodyText;
            l = Pop3Message.GetAllContents(pm);
            for (int i = 0; i < l.Count; i++)
            {
                if (String.IsNullOrEmpty(l[i].ContentDisposition.FileName) == true)
                {
                    l[i].ContentDisposition.FileName = l[i].ContentType.Name;
                }
            }
            this.AttachedFileList.DataSource = l;
        }
        private void ReceiveMailDetailButton_Click(object sender, EventArgs e)
        {
            this.ShowMailDetail();
        }
        private void ShowMailDetail()
        {
            Pop3Message pm = null;
            Int32 x = 0;

            if (Int32.TryParse(this.DetailMailIndex.Text, out x) == true)
            {
                this.zMessage = null;
                using (Pop3Client pc = new Pop3Client(this.UserName.Text, this.Password.Text, this.ServerName.Text))
                {
                    pc.Port = Int32.Parse(this.Port.Text);
                    if (pc.Authenticate() == false)
                    {
                        MessageBox.Show("Log in failure.");
                        return;
                    }

                    pm = pc.GetMessage(x);
                    if (pm.InvalidFormat == true)
                    {
                        String s = String.Format(@"{0}{1}.txt", this.OutputFilePath.Text, this.DetailMailIndex.Text);
                        System.IO.File.WriteAllText(s, pm.Data);
                        return;
                    }
                    this.SetMailData(pm);
                }
            }
        }
        private void EnsureDirectoryExist()
        {
            if (Directory.Exists(this.OutputFilePath.Text) == false)
            {
                try
                {
                    Directory.CreateDirectory(this.OutputFilePath.Text);
                }
                catch
                {
                    MessageBox.Show("Invalid directory path.");
                }
            }
        }
        private void ShowMailListButton_Click(object sender, EventArgs e)
        {
            this.SetMailList();
        }
        private void MailList_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SetMailData(this.MailList.SelectedItem as Pop3Message);
        }
        private void PreviousMailListButton_Click(object sender, EventArgs e)
        {
            this.zStartIndex -= this.zDisplayMailCount;
			this.ValidateStartIndex();
			this.SetMailList();
        }
        private void NextMailListButton_Click(object sender, EventArgs e)
        {
            this.zStartIndex += this.zDisplayMailCount;
            this.ValidateStartIndex();
            this.SetMailList();
        }
        private void MailIndex_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                this.ShowMailDetail();
            }
        }
        private void PreviousMailDetailButton_Click(object sender, EventArgs e)
        {
            Int32 x = 0;

            if (Int32.TryParse(this.DetailMailIndex.Text, out x) == true)
            {
                this.DetailMailIndex.Text = (x - 1).ToString();
                this.ShowMailDetail();
            }
        }
        private void NextMailDetailButton_Click(object sender, EventArgs e)
        {
            Int32 x = 0;

            if (Int32.TryParse(this.DetailMailIndex.Text, out x) == true)
            {
                this.DetailMailIndex.Text = (x + 1).ToString();
                this.ShowMailDetail();
            }
        }
        private void GetAttachedFileButton_Click(object sender, EventArgs e)
        {
			try
			{
				if (this.zMessage != null)
				{
                    List<Pop3Content> l = Pop3Message.GetAllContents(this.zMessage);
					for (int i = 0; i < l.Count; i++)
					{
						if (l[i].ContentDisposition.FileName == this.AttachedFileList.SelectedValue.ToString())
						{
							this.statusStrip1.Text = "Downloading...";
							String s = String.Format(@"{0}{1}", this.OutputFilePath.Text, l[i].ContentDisposition.FileName);
							this.EnsureDirectoryExist();
							l[i].DecodeData(s);
							break;
						}
					}
				}
			}
			catch
			{
				this.statusStrip1.Text = "";
			}
        }
        private void Port_Validating(object sender, CancelEventArgs e)
        {
            Int32 x = 0;
            if (Int32.TryParse(this.Port.Text, out x) == true)
            {
                if (x > 0)
                {
                    this.Port.Text = x.ToString();
                    return;
                }
            }
            MessageBox.Show("A positive interger required.");
        }
        private void ListMailIndex_Validating(object sender, CancelEventArgs e)
        {
            Int32 x = 0;
            if (Int32.TryParse(this.ListMailIndex.Text, out x) == true)
            {
                if (x > 0)
                {
                    this.ListMailIndex.Text = x.ToString();
                    this.zStartIndex = x;
                    this.ValidateStartIndex();
                    return;
                }
            }
            MessageBox.Show("A positive interger required.");
        }
        private void DetailMailIndex_Validating(object sender, CancelEventArgs e)
        {
            Int32 x = 0;
            if (Int32.TryParse(this.DetailMailIndex.Text, out x) == true)
            {
                if (x > 0)
                {
                    this.DetailMailIndex.Text = x.ToString();
                    return;
                }
            }
            MessageBox.Show("A positive interger required.");
        }
        private void ValidateStartIndex()
        {
            if (this.zStartIndex < this.zDisplayMailCount)
            {
                this.zStartIndex = this.zDisplayMailCount;
            }
            if (this.zStartIndex > this.zTotalMailCount)
            {
                this.zStartIndex = this.zTotalMailCount;
            }
        }
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.Save();
        }
        private void DebugButton_Click(object sender, EventArgs e)
        {
            Pop3Message mg = new Pop3Message(File.ReadAllText(@"C:\Mail1.txt"));
            String s = mg.Subject;
            s = mg.BodyText;
            return;

        }
    }
}