﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using Higuchi.Net.Mail;

namespace Higuchi.Net.Pop3
{
	public class Pop3Server : Server
	{
		private IMailDataProvider _DataProvider = new NullDataProvider();
		public IMailDataProvider DataProvider
		{
			get { return _DataProvider; }
			set { _DataProvider = value; }
		}
		public Pop3Server(IPAddress address, Int32 port, IMailDataProvider provider)
			: base(address, port)
		{
			_DataProvider = provider;
		}
		/// <summary>
		/// This method is called as separated thread.
		/// </summary>
		/// <param name="socket"></param>
		protected override void OnConnected(Socket socket)
		{
			Pop3Processor p = new Pop3Processor(this, socket);
			p.Authenticated += (o, e) => this.OnOutputText(e.UserName);
			p.ReceiveMessage += (o, e) => this.OnOutputText(e.Text);
			p.SendMessage += (o, e) => this.OnOutputText(e.Text);
			p.Error += (o, e) => this.OnOutputText(e.GetException().ToString());
			p.Start();
		}
		public static String CreateMessage(Boolean ok, String text)
		{
			if (ok == true)
			{
				return "+OK " + text;
			}
			return "+ERR " + text;
		}
		public static String CreateMessage(Pop3ResponseMessage message)
		{
			switch (message)
			{
				case Pop3ResponseMessage.UserNameReceived: return "+OK your password please.";
				case Pop3ResponseMessage.Authenticated: return "+OK authenticated.server is waiting for your command...";
				case Pop3ResponseMessage.AuthenticateFailure: return "+ERR not authenticated.send your username again.";
				case Pop3ResponseMessage.NoopMessage: return "+OK Hello,Hello!";
				case Pop3ResponseMessage.CommandIsInvalidFormat: return "+ERR command is invalid format.";
			}
			return "command is invalid.";
		}
		public static Pop3RequestCommand GetRequestCommand(String text)
		{
			if (String.IsNullOrEmpty(text) == true) { return Pop3RequestCommand.Unkown; }
			String s = text.ToUpper();
			if (s.StartsWith("USER ") == true) { return Pop3RequestCommand.User; }
			if (s.StartsWith("PASS ") == true) { return Pop3RequestCommand.Pass; }
			if (s == "LIST" || s.StartsWith("LIST ") == true) { return Pop3RequestCommand.List; }
			if (s == "UIDL" || s.StartsWith("UIDL ") == true) { return Pop3RequestCommand.Uidl; }
			if (s == "STAT") { return Pop3RequestCommand.Stat; }
			if (s == "RETR" || s.StartsWith("RETR ") == true) { return Pop3RequestCommand.Retr; }
			if (s == "TOP" || s.StartsWith("TOP ") == true) { return Pop3RequestCommand.Top; }
			if (s == "NOOP") { return Pop3RequestCommand.Noop; }
			if (s == "QUIT") { return Pop3RequestCommand.Quit; }
			if (s == "RSET") { return Pop3RequestCommand.Rset; }
			return Pop3RequestCommand.Unkown;
		}
	}
}
