﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Higuchi.Net.Pop3
{
	public class AuthenticatedState : Pop3ProcessState
	{
		private String _UserName = "";
		private String _Password = "";
		private List<Int32> _DeleteIndexList = new List<int>();
		public String UserName
		{
			get { return _UserName; }
		}
		public String Password
		{
			get { return _Password; }
		}
		public AuthenticatedState(Pop3Processor processer, String userName, String password)
		{
			_Processor = processer;
			_UserName = userName;
			_Password = password;
		}
		public override Pop3ProcessState ReceiveClientMessage(string text)
		{
			Pop3ProcessState state = this;
			Pop3RequestCommand cm = Pop3Server.GetRequestCommand(text);
			Int32 index = 0;

			if (this.ValidateCommand(cm) == true)
			{
				if (cm == Pop3RequestCommand.List)
				{
					state.ServerMessage = _Processor.DataProvider.List(_UserName);
				}
				else if (cm == Pop3RequestCommand.Uidl)
				{
					state.ServerMessage = _Processor.DataProvider.Uidl(_UserName);
				}
				else if (cm == Pop3RequestCommand.Stat)
				{
					state.ServerMessage = _Processor.DataProvider.Stat(_UserName);
				}
				else if (cm == Pop3RequestCommand.Retr)
				{
					if (Int32.TryParse(text.Remove(0, 5), out index) == true)
					{
						state.ServerMessage = _Processor.DataProvider.Retr(_UserName, index);
					}
					else
					{
						state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.CommandIsInvalidFormat);
					}
				}
				else if (cm == Pop3RequestCommand.Top)
				{
				}
				else if (cm == Pop3RequestCommand.Dele)
				{
					if (Int32.TryParse(text.Remove(0, 5), out index) == true)
					{
						if (_DeleteIndexList.Contains(index) == false)
						{
							_DeleteIndexList.Add(index);
						}
						state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.CommandIsExecuted);
					}
					else
					{
						state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.CommandIsInvalidFormat);
					}
				}
				else if (cm == Pop3RequestCommand.Rset)
				{
					_DeleteIndexList.Clear();
					state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.CommandIsExecuted);
				}
				else if (cm == Pop3RequestCommand.Noop)
				{
					state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.NoopMessage);
				}
				else if (cm == Pop3RequestCommand.Quit)
				{
					_Processor.DataProvider.Dele(_UserName, _DeleteIndexList);
					state = new QuitState();
				}
			}
			else
			{
				state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.CommandIsInvalidFormat);
			}
			return state;
		}
		private Boolean ValidateCommand(Pop3RequestCommand command)
		{
			switch (command)
			{
				case Pop3RequestCommand.List: return true;
				case Pop3RequestCommand.Uidl: return true;
				case Pop3RequestCommand.Stat: return true;
				case Pop3RequestCommand.Retr: return true;
				case Pop3RequestCommand.Top: return true;
				case Pop3RequestCommand.Dele: return true;
				case Pop3RequestCommand.Rset: return true;
				case Pop3RequestCommand.Noop: return true;
				case Pop3RequestCommand.Quit: return true;
			}
			return false;
		}
	}
}
