﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Higuchi.Net.Pop3
{
	public class ServerReadyState : Pop3ProcessState
	{
		private Boolean _Authenticated = false;
		private Boolean _ReceiveUserName = false;
		private String _UserName = "";
		private String _Password = "";
		private String _TimeStamp = Guid.NewGuid().ToString();
		public String TimeStamp
		{
			get { return _TimeStamp; }
		}
		public ServerReadyState(Pop3Processor processor)
		{
			_Processor = processor;
		}
		public ServerReadyState(Pop3Processor processor, Boolean authenticated)
		{
			_Processor = processor;
			_Authenticated = authenticated;
		}
		public override Pop3ProcessState ReceiveClientMessage(String text)
		{
			Pop3ProcessState state = this;
			Pop3RequestCommand cm = Pop3Server.GetRequestCommand(text);

			if (this.ValidateCommand(cm) == true)
			{
				if (cm == Pop3RequestCommand.User)
				{
					//text is USER some_username,so remove first 5 char.
					_UserName = text.Remove(0, 5);
					_ReceiveUserName = true;
					state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.UserNameReceived);
				}
				else if (cm == Pop3RequestCommand.Pass)
				{
					_Password = text.Remove(0, 5);
					if (_Processor.DataProvider.Authenticate(_UserName, _Password) == true ||
						_Processor.DataProvider.AuthenticateAPop(_UserName, _Password, _TimeStamp) == true)
					{
						state = new AuthenticatedState(_Processor, _UserName, _Password);
						state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.Authenticated);
					}
					else
					{
						_UserName = "";
						_Password = "";
						_ReceiveUserName = false;
						state.ServerMessage = Pop3Server.CreateMessage(Pop3ResponseMessage.AuthenticateFailure);
					}
				}
			}
			else
			{
				state.ServerMessage = Pop3Server.CreateMessage(false, "Bad command sequence.");
			}
			return state;
		}
		private Boolean ValidateCommand(Pop3RequestCommand command)
		{
			switch (command)
			{
				case Pop3RequestCommand.User: return _ReceiveUserName == false;
				case Pop3RequestCommand.Pass: return _ReceiveUserName == true;
				case Pop3RequestCommand.Noop: return true;
				case Pop3RequestCommand.Quit: return true;
			}
			return false;
		}
	}
}
