﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Net.NetworkInformation;

namespace Higuchi.Net
{
	public class Server
	{
		public event EventHandler<SocketMessageEventArgs> OutputText;
		private readonly IPAddress _IPAddress;
		private readonly Int32 _Port;
		private IPAddress _DnsServerAddress = null;
		public IPAddress IPAddress
		{
			get { return _IPAddress; }
		}
		public Int32 Port
		{
			get { return _Port; }
		}
		public IPAddress DnsServerAddress
		{
			get { return _DnsServerAddress; }
		}
		private TcpListener _TcpListener = null;
		public Server(IPAddress address, Int32 port)
		{
			_IPAddress = address;
			_Port = port;
			_DnsServerAddress = Server.GetDnsServer();
			this.OutputText += (o, e) => this.WriteLineToConsole(e.Text);
		}
		public void Start()
		{
			try
			{
				_TcpListener = new TcpListener(_IPAddress, _Port);
				_TcpListener.Start();

				this.OnOutputText("Server is ready for listening...");
				Thread thd = new Thread(new ThreadStart(this.StartListen));
				thd.Start();
			}
			catch (Exception e)
			{
				this.OnOutputText("Error start listening process : " + e.ToString());
			}
		}
		private void StartListen()
		{
			try
			{
				while (true)
				{
					Socket sc = _TcpListener.AcceptSocket();
					sc.Blocking = true;
					if (sc.Connected)
					{
						this.OnOutputText("Client connected : {0}", sc.RemoteEndPoint);
						Thread thd = new Thread(() => this.OnConnected(sc));
						thd.IsBackground = true;
						thd.Start();
					}
				}
			}
			catch (SocketException ex)
			{
				this.OnOutputText("SocketException : {0}", ex);
			}
		}
		public static IPAddress GetLocalIPAddress(Socket socket)
		{
			var p = socket.LocalEndPoint as IPEndPoint;
			if (p == null) { return null; }
			return p.Address;
		}
		public static IPAddress GetRemoteIPAddress(Socket socket)
		{
			var p = socket.RemoteEndPoint as IPEndPoint;
			if (p == null) { return null; }
			return p.Address;
		}
		/// <summary>
		/// Call separated thread.
		/// </summary>
		/// <param name="socket"></param>
		protected virtual void OnConnected(Socket socket)
		{
		}
		public void Stop()
		{
			_TcpListener.Stop();
		}
		protected void OnOutputText(String text, params Object[] args)
		{
			if (this.OutputText != null)
			{
				this.OutputText(this, new SocketMessageEventArgs(String.Format(text, args)));
			}
		}
		protected void WriteLineToConsole(String text)
		{
			Console.WriteLine(text);
		}
		protected void WriteLineToConsole(String text, params Object[] args)
		{
			Console.WriteLine(text, args);
		}
		public static IPAddress GetDnsServer()
		{
			NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();
			foreach (NetworkInterface ni in nics)
			{
				if (ni.OperationalStatus == OperationalStatus.Up)
				{
					IPAddressCollection ips = ni.GetIPProperties().DnsAddresses;
					foreach (IPAddress ip in ips)
					{
						return ip;
					}
				}
			}
			return null;
		}
	}
}
