﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using Higuchi.Net.Mail;
using Bdev.Net.Dns;

namespace Higuchi.Net.Smtp
{
	internal class DataReceiveState : SmtpProcessState
	{
		private Boolean _Authenticated = false;
		private MailAddress _MailFrom = null;
		private List<MailAddress> _RcptToList = null;
		private StringBuilder _Data = new StringBuilder();
		public DataReceiveState(SmtpProcessor processor, MailAddress mailFrom, List<MailAddress> RcptToList)
		{
			_Processor = processor;
			_MailFrom = mailFrom;
			_RcptToList = RcptToList;
		}
		public DataReceiveState(SmtpProcessor processor, MailAddress mailFrom, List<MailAddress> RcptToList, Boolean authenticated)
		{
			_Processor = processor;
			_MailFrom = mailFrom;
			_RcptToList = RcptToList;
			_Authenticated = authenticated;
		}
		public override SmtpProcessState ReceiveClientMessage(string text)
		{
			SmtpProcessState state = this;
			SmtpRequestCommand cm = SmtpServer.GetRequestCommand(text);

			_Data.AppendLine(text);
			if (text != ".") { return state; }

			for (int i = 0; i < _RcptToList.Count; i++)
			{
				if (this._Processor.IsDelivery(_RcptToList[i]) == true)
				{
					_Processor.DataProvider.Save(_RcptToList[i].UserName, _Data.ToString());
				}
				else if (_Processor.CanRelay(_Authenticated) == true)
				{
					this.RelayMail(_RcptToList[i]);
				}
			}

			state = new ServerReadyState(_Processor);
			state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.ServiceReady);
			return state;
		}
		private Boolean RelayMail(MailAddress rcptTo)
		{
			SmtpClient cl = new SmtpClient();
			List<MXRecord> l = null;
			SendMailResult rs = null;

			l = this.GetMXRecord(rcptTo.DomainName);
			l.Sort((x, y) => Comparer<Int32>.Default.Compare(x.Preference, y.Preference));
			for (int i = 0; i < l.Count; i++)
			{
				cl.HostName = Dns.GetHostName();
				cl.ServerName = l[i].DomainName;
				cl.Port = 25;
				cl.AuthenticateMode = SmtpAuthenticateMode.None;
				rs = cl.SendMail(_MailFrom.ToString(), rcptTo.ToString(), "", "", _Data.ToString());
				if (rs.SendSuccessful == true)
				{ return true; }
			}
			return false;
		}
		private List<MXRecord> GetMXRecord(String domainName)
		{
			try
			{
				return Resolver.MXLookup(domainName, _Processor.DnsServerAddress).ToList();
			}
			catch { }
			return new List<MXRecord>();
		}
	}
}
