﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Higuchi.Net.Mail;

namespace Higuchi.Net.Smtp
{
	public class EnvelopeState : SmtpProcessState
	{
		private Boolean _Authenticated = false;
		private MailAddress _MailFrom = null;
		private List<MailAddress> _RcptToList = new List<MailAddress>();
		public Boolean Authenticated
		{
			get { return _Authenticated; }
		}
		public MailAddress MailFrom
		{
			get { return _MailFrom; }
		}
		public EnvelopeState(SmtpProcessor processor, MailAddress mailFrom)
		{
			_Processor = processor;
			_MailFrom = mailFrom;
		}
		public EnvelopeState(SmtpProcessor processor, MailAddress mailFrom, Boolean authenticated)
		{
			_Processor = processor;
			_MailFrom = mailFrom;
			_Authenticated = authenticated;
		}
		public override SmtpProcessState ReceiveClientMessage(String text)
		{
			SmtpProcessState state = this;
			SmtpRequestCommand cm = SmtpServer.GetRequestCommand(text);
			MailAddress m = null;

			if (this.IsRsetOrQuit(cm) == true) { return this.CreateState(cm); }

			if (this.ValidateCommand(cm) == true)
			{
				if (cm == SmtpRequestCommand.Rcpt)
				{
					m = this.ParseRcptTo(text);
					if (this._Processor.IsDelivery(m) == true)
					{
						_RcptToList.Add(m);
						state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.RequestedMailActionOkay_Completed);
					}
					else
					{
						if (this._Processor.CanRelay(this.Authenticated) == true)
						{
							_RcptToList.Add(m);
							state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.RequestedMailActionOkay_Completed);
						}
						else
						{
							state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.TransactionFailed);
						}
					}
				}
				else if (cm == SmtpRequestCommand.Data)
				{
					state = new DataReceiveState(_Processor, _MailFrom, _RcptToList, _Authenticated);
					state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.StartMailInput);
				}
			}
			else
			{
				state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.SyntaxError_CommandUnrecognized);
			}
			return state;
		}
		private Boolean ValidateCommand(SmtpRequestCommand command)
		{
			return command == SmtpRequestCommand.Rcpt || command == SmtpRequestCommand.Data;
		}
		private MailAddress ParseRcptTo(String text)
		{
			//RCPT TO: --> remove first 8 charactar
			String s = text.Remove(0, 8).Replace(Environment.NewLine, "");
			return new MailAddress(s);
		}
	}
}
