﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Higuchi.Net.Smtp
{
	internal class LoginAuthenticatingState : SmtpProcessState
	{
		private String _UserName = null;
		public LoginAuthenticatingState(SmtpProcessor processor)
		{
			_Processor = processor;
		}
		public override SmtpProcessState ReceiveClientMessage(string text)
		{
			SmtpProcessState state = this;
			String s = this.FromBase64String(text);
			SmtpRequestCommand cm = SmtpServer.GetRequestCommand(text);

			if (this.IsRsetOrQuit(cm) == true) { return this.CreateState(cm); }
			
			if (_UserName == null)
			{
				if (s == null)
				{
					state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.SyntaxError_CommandUnrecognized);
				}
				else
				{
					_UserName = s;
					state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.WaitingForAuthentication);
				}
			}
			else 
			{
				if (_Processor.DataProvider.Authenticate(_UserName, s) == true)
				{
					state = new ServerReadyState(_Processor);
					state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.AuthenticationSuccessful);
				}
				else
				{
					state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.TemporaryAuthenticationFailure);
				}
			}
			return state;
		}
		private String FromBase64String(String text)
		{
			try
			{
				return Mail.MailParser.FromBase64String(text);
			}
			catch { }
			return null;
		}
	}
}
