﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using Higuchi.Net.Mail;

namespace Higuchi.Net.Smtp
{
	internal class ServerReadyState : SmtpProcessState
	{
		private Boolean _Authenticated = false;
		public ServerReadyState(SmtpProcessor processor)
		{
			_Processor = processor;
		}
		public ServerReadyState(SmtpProcessor processor, Boolean authenticated)
		{
			_Processor = processor;
			_Authenticated = authenticated;
		}
		public override SmtpProcessState ReceiveClientMessage(String text)
		{
			SmtpProcessState state = this;
			SmtpRequestCommand cm = SmtpServer.GetRequestCommand(text);

			if (this.IsRsetOrQuit(cm) == true) { return this.CreateState(cm); }

			if (this.ValidateCommand(cm) == true)
			{
				if (cm == SmtpRequestCommand.Auth)
				{
					String[] ss = text.Split(' ');
					if (ss.Length > 1)
					{
						var mode = this.GetAuthenticateMode(ss[1]);
						switch (mode)
						{
							case SmtpAuthenticateMode.Plain: state = new PlainAuthenticatingState(_Processor); break;
							case SmtpAuthenticateMode.Login: state = new LoginAuthenticatingState(_Processor); break;
							case SmtpAuthenticateMode.Cram_MD5: state = new CramMD5AuthenticatingState(_Processor); break;
						}
						state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.WaitingForAuthentication);
					}
					else
					{
						state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.SyntaxErrorInParametersOrArguments);
					}
				}
				else if (cm == SmtpRequestCommand.Mail)
				{
					MailAddress m = this.GetMailFrom(text);
					if (m == null)
					{
						state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.SyntaxError_CommandUnrecognized);
					}
					else
					{
						if (_Processor.CanRelay(_Authenticated) == true ||
							_Processor.DataProvider.IsConnect(m.UserName) == true)
						{
							state = new EnvelopeState(_Processor, m, _Authenticated);
							state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.RequestedMailActionOkay_Completed);
						}
						else
						{
							state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.AuthenticationRequired);
						}
					}
				}
				else if (cm == SmtpRequestCommand.Quit)
				{
					state = new QuitState();
					state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.ServiceClosingTransmissionChannel);
				}
				else
				{
					this.ReceiveClientMessage(state, cm);
				}
			}
			else
			{
				state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.SyntaxError_CommandUnrecognized);
			}
			return state;
		}
		private SmtpAuthenticateMode GetAuthenticateMode(String text)
		{
			switch (text.ToUpper())
			{
				case "AUTO": return SmtpAuthenticateMode.Auto;
				case "CRAM-MD5": return SmtpAuthenticateMode.Cram_MD5;
				case "LOGIN": return SmtpAuthenticateMode.Login;
				case "PLAIN": return SmtpAuthenticateMode.Plain;
			}
			return SmtpAuthenticateMode.None;
		}
		private MailAddress GetMailFrom(String text)
		{
			//Mail FROM: --> remove first 10 charactar
			String s = text.Remove(0, 10).Replace(Environment.NewLine, "");
			return MailAddress.TryCreate(s);
		}
		private Boolean ValidateCommand(SmtpRequestCommand command)
		{
			switch (command)
			{
				case SmtpRequestCommand.Helo: return true;
				case SmtpRequestCommand.Ehlo: return true;
				case SmtpRequestCommand.Help: return true;
				case SmtpRequestCommand.Auth: return true;
				case SmtpRequestCommand.Noop: return true;
				case SmtpRequestCommand.Mail: return true;
				case SmtpRequestCommand.Quit: return true;
			}
			return false;
		}
	}
}
