﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace Higuchi.Net.Smtp
{
	public abstract class SmtpProcessState
	{
		protected SmtpProcessor _Processor = null;
		private String _ServerMessage = "";
		public String ServerMessage
		{
			get { return _ServerMessage; }
			set { _ServerMessage = value; }
		}
		public abstract SmtpProcessState ReceiveClientMessage(String text);
		/// <summary>
		/// Process that state does not change request command.
		/// </summary>
		/// <param name="state"></param>
		/// <param name="command"></param>
		protected void ReceiveClientMessage(SmtpProcessState state, SmtpRequestCommand command)
		{
			var cm = command;
			if (cm == SmtpRequestCommand.Help)
			{
				state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.HelpMessage);
			}
			else if (cm == SmtpRequestCommand.Helo || cm == SmtpRequestCommand.Ehlo)
			{
				state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.RequestedMailActionOkay_Completed);
			}
			else if (command == SmtpRequestCommand.Noop)
			{
				state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.RequestedMailActionOkay_Completed);
			}
		}
		protected Boolean IsRsetOrQuit(SmtpRequestCommand command)
		{
			return command == SmtpRequestCommand.Rset || 
				command == SmtpRequestCommand.Quit;
		}
		protected SmtpProcessState CreateState(SmtpRequestCommand command)
		{
			SmtpProcessState state = null;
			if (command == SmtpRequestCommand.Rset)
			{
				state = new ServerReadyState(_Processor);
				state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.RequestedMailActionOkay_Completed);
			}
			else if (command == SmtpRequestCommand.Quit)
			{
				state = new QuitState();
				state.ServerMessage = SmtpServer.CreateMessage(SmtpCommandResultCode.ServiceClosingTransmissionChannel);
			}
			return state;
		}
	}
}
