﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Higuchi.Net;
using Higuchi.Net.Mail;
using Higuchi.Net.Pop3;
using Higuchi.Net.Smtp;

namespace MailTest
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class MailTest
    {
        private String _FolderPath = "../../../MailTest/TestData/";
        public MailTest()
        {
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        private Pop3Message CreatePop3Message(Int32 TestNo)
        {
            String path = "";
            Pop3Message mg = null;

            path = String.Format("{0}Mail{1:000}.txt", _FolderPath, TestNo);
            mg = new Pop3Message(File.ReadAllText(path));

            return mg;
        }
        [TestMethod]
        public void Test001()
        {
            var mg = this.CreatePop3Message(1);

            Assert.AreEqual("Rytinės finansų rinkų naujienos", mg.Subject);
            Assert.AreEqual("<higty@csharpmail.com>", mg.To);
            Assert.AreEqual("b1_5508506f9ba50f2d830b92f2e2d34471", mg.MultiPartBoundary);
            Assert.AreEqual("multipart/alternative", mg.ContentType.Value);
        }
        [TestMethod]
        public void Test002()
        {
            var mg = this.CreatePop3Message(2);

            Assert.AreEqual("テストメール(TestMail)", mg.Subject);
            Assert.AreEqual("日本語ファイル名.txt", mg.Contents[1].FileName);
            Assert.AreEqual(false, mg.Contents[0].IsAttachment);
            Assert.AreEqual("multipart/mixed", mg.ContentType.Value);
        }
        /// <summary>
        /// RCF2231のテスト
        /// </summary>
        [TestMethod]
        public void Test003()
        {
            ContentType ctp = null;
            ContentDisposition cd = null;

            ctp = new ContentType("");
            MailParser.ParseContentType(ctp, "Content-Type: text/plain;" + Environment.NewLine
                + "name=\"=?iso-2022-jp?B?GyRCRnxLXDhsJVUlISUkJWtMPhsoQi50eHQ=?=\"");
            Assert.AreEqual("日本語ファイル名.txt", ctp.Name);

            ctp = new ContentType("");
            MailParser.ParseContentType(ctp, "Content-Type: text/plain;" + Environment.NewLine
                + "name*=iso-2022-jp''%1B%24BF%7CK%5C8l%25U%25%21%25%24%25kL%3E%1B%28B%20copy.txt");
            Assert.AreEqual("日本語ファイル名 copy.txt", ctp.Name);


            cd = new ContentDisposition("");
            MailParser.ParseContentDisposition(cd, "Content-Disposition: attachment;" + Environment.NewLine
                + "filename*=iso-2022-jp''%1B%24BF%7CK%5C8l%25U%25%21%25%24%25kL%3E%1B%28B%20copy.txt");
            Assert.AreEqual("日本語ファイル名 copy.txt", cd.FileName);

            cd = new ContentDisposition("");
            MailParser.ParseContentDisposition(cd, "Content-Disposition: attachment;" + Environment.NewLine
                + "\tfilename*0=MailTe" + Environment.NewLine
                + "\tfilename*1=st.txt");
            Assert.AreEqual("MailTest.txt", cd.FileName);

            cd = new ContentDisposition("");
            MailParser.ParseContentDisposition(cd, "Content-Disposition: attachment;" + Environment.NewLine
                + "\tfilename*0*=iso-2022-jp''%1B%24BF%7CK%5C8l%25U%25%21%25%24" + Environment.NewLine
                + "\tfilename*1*=%25kL%3E%1B%28B%20copy.txt");
            Assert.AreEqual("日本語ファイル名 copy.txt", cd.FileName);

        }
        [TestMethod]
        public void Test004()
        {
            Encoding en = null;

            en = Encoding.GetEncoding("iso-2022-jp");
            Assert.AreEqual("name*=iso-2022-jp''%1B%24BF%7CK%5C8l%25U%25%21%25%24%25k%1B%28B"
                , MailParser.EncodeToMailHeaderLineByRfc2231("name", "日本語ファイル", en, 70));

        }
        [TestMethod]
        public void Test005()
        {
            SmtpContent ct = new SmtpContent();
            Encoding en = Encoding.GetEncoding("iso-2022-JP");

            ct.FieldParameterEncoding = FieldParameterEncoding.Rfc2231;
            ct.FileName = "添付ファイルと改行のチェックをテスト";

            String text = ct.GetDataText();

            Assert.AreEqual("filename*=iso-2022-jp''%1B%24BE%3AIU%25U%25%21%25%24%25k%1B%28B"
                , MailParser.EncodeToMailHeaderLineByRfc2231("filename", "添付ファイル", en, 72));

            Assert.AreEqual(@"filename*0*=iso-2022-jp''%1B%24BE%3AIU%25U%25%21%25%24%25k%24H2%7E9T%24N%25A%25%27%25C%25%
 filename*1*=2F%24r%25F%259%25H%1B%28B;"
                , MailParser.EncodeToMailHeaderLineByRfc2231("filename", "添付ファイルと改行のチェックをテスト", en, 72));

        }
		[TestMethod]
		public void Test006()
		{
			var mg = this.CreatePop3Message(3);

			Assert.AreEqual("http://www.mysite.com.br/assinatura/Logo_Nickpel.jpg" + Environment.NewLine
				, mg.Contents[1].BodyText);
		}
        [TestMethod]
        public void Test007()
        {
            var mg = new SmtpMessage();
            mg.ContentTransferEncoding = TransferEncoding.QuotedPrintable;
            mg.BodyText = "." + Environment.NewLine + ".." + Environment.NewLine + "...Comment" + Environment.NewLine;
            String body = mg.GetDataText();
            var mgg = new Pop3Message(body);

            Assert.AreEqual(mgg.BodyText, mgg.BodyText);
        }
        [TestMethod]
        public void Test008()
        {
            var mg = this.CreatePop3Message(4);

            Assert.AreEqual(Encoding.GetEncoding("Big5"), mg.ContentEncoding);
        }
    }
}
